@@ -1,7 +1,6 @@
 from __future__ import absolute_import
 from __future__ import division
 
-import inspect
 import logging
 import os
 import re
@@ -13,10 +12,10 @@ import tarfile
 import tempfile
 import threading
 import time
-import types
 
 from pwnlib import term
 from pwnlib.context import context, LocalContext
+from pwnlib.exception import PwnlibException
 from pwnlib.log import Logger
 from pwnlib.log import getLogger
 from pwnlib.term import text
@@ -123,7 +122,7 @@ class ssh_channel(sock):
                             pass
 
                 self.resizer = resizer
-                term.term.on_winch.append(self.resizer)
+                term.term.on_winch.append(self.resizer)  # XXX memory leak
             else:
                 self.resizer = None
 
@@ -323,6 +322,7 @@ class ssh_process(ssh_channel):
 
         for lib in maps:
             remote_path = lib.split(self.parent.host)[-1]
+            remote_path = self.parent.readlink('-f', remote_path).decode()
             for line in maps_raw.splitlines():
                 if line.endswith(remote_path):
                     address = line.split('-')[0]
@@ -340,6 +340,7 @@ class ssh_process(ssh_channel):
         automatically.
 
         Examples:
+
             >>> s =  ssh(host='example.pwnme')
             >>> p = s.process('true')
             >>> p.libc  # doctest: +ELLIPSIS
@@ -383,6 +384,7 @@ class ssh_process(ssh_channel):
         r"""Retrieve the address of an environment variable in the remote process.
 
         Examples:
+
             >>> s = ssh(host='example.pwnme')
             >>> p = s.process(['python', '-c', 'import time; time.sleep(10)'])
             >>> hex(p.getenv('PATH'))  # doctest: +ELLIPSIS
@@ -542,24 +544,37 @@ class ssh(Timeout, Logger):
     #: Remote port (``int``)
     port = None
 
+    #: Remote username (``str``)
+    user = None
+
+    #: Remote password (``str``)
+    password = None
+
+    #: Remote private key (``str``)
+    key = None
+
+    #: Remote private key file (``str``)
+    keyfile = None
+
     #: Enable caching of SSH downloads (``bool``)
     cache = True
 
+    #: Enable raw mode and don't probe the environment (``bool``)
+    raw = False
+
     #: Paramiko SSHClient which backs this object
     client = None
 
-    #: Paramiko SFTPClient object which is used for file transfers.
-    #: Set to :const:`None` to disable ``sftp``.
-    sftp = None
-
     #: PID of the remote ``sshd`` process servicing this connection.
     pid = None
 
     _cwd = '.'
+    _tried_sftp = False
 
     def __init__(self, user=None, host=None, port=22, password=None, key=None,
                  keyfile=None, proxy_command=None, proxy_sock=None, level=None,
-                 cache=True, ssh_agent=False, ignore_config=False, raw=False, *a, **kw):
+                 cache=True, ssh_agent=False, ignore_config=False, raw=False, 
+                 auth_none=False, *a, **kw):
         """Creates a new ssh connection.
 
         Arguments:
@@ -573,10 +588,11 @@ class ssh(Timeout, Logger):
             proxy_sock(str): Use this socket instead of connecting to the host.
             timeout: Timeout, in seconds
             level: Log level
-            cache: Cache downloaded files (by hash/size/timestamp)
-            ssh_agent: If :const:`True`, enable usage of keys via ssh-agent
-            ignore_config: If :const:`True`, disable usage of ~/.ssh/config and ~/.ssh/authorized_keys
-            raw: If :const:`True`, assume a non-standard shell and don't probe the environment
+            cache(bool): Cache downloaded files (by hash/size/timestamp)
+            ssh_agent(bool): If :const:`True`, enable usage of keys via ssh-agent
+            ignore_config(bool): If :const:`True`, disable usage of ~/.ssh/config and ~/.ssh/authorized_keys
+            raw(bool): If :const:`True`, assume a non-standard shell and don't probe the environment
+            auth_none(bool): If :const:`True`, try to authenticate with no authentication methods
 
         NOTE: The proxy_command and proxy_sock arguments is only available if a
         fairly new version of paramiko is used.
@@ -613,6 +629,9 @@ class ssh(Timeout, Logger):
         self._platform_info = {}
         self._aslr = None
         self._aslr_ulimit = None
+        self._cpuinfo_cache = None
+        self._user_shstk = None
+        self._ibt = None
 
         misc.mkdir_p(self._cachedir)
 
@@ -669,6 +688,11 @@ class ssh(Timeout, Logger):
                            "    To remove the existing entry from your known_hosts and trust the new key, run the following commands:\n"
                            "        $ ssh-keygen -R %(host)s\n"
                            "        $ ssh-keygen -R [%(host)s]:%(port)s" % locals())
+            except paramiko.SSHException as e:
+                if user and auth_none and str(e) == "No authentication methods available":
+                    self.client.get_transport().auth_none(user)
+                else:
+                    raise
 
             self.transport = self.client.get_transport()
             self.transport.use_compression(True)
@@ -714,6 +738,9 @@ class ssh(Timeout, Logger):
 
     @property
     def sftp(self):
+        """Paramiko SFTPClient object which is used for file transfers.
+        Set to :const:`None` to disable ``sftp``.
+        """
         if not self._tried_sftp:
             try:
                 self._sftp = self.transport.open_sftp_client()
@@ -748,6 +775,7 @@ class ssh(Timeout, Logger):
             Return a :class:`pwnlib.tubes.ssh.ssh_channel` object.
 
         Examples:
+
             >>> s =  ssh(host='example.pwnme')
             >>> sh = s.shell('/bin/sh')
             >>> sh.sendline(b'echo Hello; exit')
@@ -756,7 +784,7 @@ class ssh(Timeout, Logger):
         """
         return self.run(shell, tty, timeout = timeout)
 
-    def process(self, argv=None, executable=None, tty=True, cwd=None, env=None, timeout=Timeout.default, run=True,
+    def process(self, argv=None, executable=None, tty=True, cwd=None, env=None, ignore_environ=None, timeout=Timeout.default, run=True,
                 stdin=0, stdout=1, stderr=2, preexec_fn=None, preexec_args=(), raw=True, aslr=None, setuid=None,
                 shell=False):
         r"""
@@ -784,8 +812,9 @@ class ssh(Timeout, Logger):
                 Working directory.  If :const:`None`, uses the working directory specified
                 on :attr:`cwd` or set via :meth:`set_working_directory`.
             env(dict):
-                Environment variables to set in the child.  If :const:`None`, inherits the
-                default environment.
+                Environment variables to add to the environment.
+            ignore_environ(bool):
+                Ignore default environment.  By default use default environment iff env not specified.
             timeout(int):
                 Timeout to set on the `tube` created to interact with the process.
             run(bool):
@@ -825,6 +854,7 @@ class ssh(Timeout, Logger):
             Requires Python on the remote server.
 
         Examples:
+
             >>> s = ssh(host='example.pwnme')
             >>> sh = s.process('/bin/sh', env={'PS1':''})
             >>> sh.sendline(b'echo Hello; exit')
@@ -900,188 +930,12 @@ class ssh(Timeout, Logger):
             b'$ \n'
 
         """
-        if not argv and not executable:
-            self.error("Must specify argv or executable")
+        cwd = cwd or self.cwd
+        script = misc._create_execve_script(argv=argv, executable=executable,
+                cwd=cwd, env=env, stdin=stdin, stdout=stdout, stderr=stderr,
+                ignore_environ=ignore_environ, preexec_fn=preexec_fn, preexec_args=preexec_args,
+                aslr=aslr, setuid=setuid, shell=shell, log=self)
 
-        aslr      = aslr if aslr is not None else context.aslr
-
-        argv, env = misc.normalize_argv_env(argv, env, self)
-
-        if shell:
-            if len(argv) != 1:
-                self.error('Cannot provide more than 1 argument if shell=True')
-            argv = [bytearray(b'/bin/sh'), bytearray(b'-c')] + argv
-
-        executable = executable or argv[0]
-        cwd        = cwd or self.cwd
-
-        # Validate, since failures on the remote side will suck.
-        if not isinstance(executable, (six.text_type, six.binary_type, bytearray)):
-            self.error("executable / argv[0] must be a string: %r" % executable)
-        executable = bytearray(packing._need_bytes(executable, min_wrong=0x80))
-
-        # Allow passing in sys.stdin/stdout/stderr objects
-        handles = {sys.stdin: 0, sys.stdout:1, sys.stderr:2}
-        stdin  = handles.get(stdin, stdin)
-        stdout = handles.get(stdout, stdout)
-        stderr = handles.get(stderr, stderr)
-
-        # Allow the user to provide a self-contained function to run
-        def func(): pass
-        func      = preexec_fn or func
-        func_args = preexec_args
-
-        if not isinstance(func, types.FunctionType):
-            self.error("preexec_fn must be a function")
-
-        func_name = func.__name__
-        if func_name == (lambda: 0).__name__:
-            self.error("preexec_fn cannot be a lambda")
-
-        func_src  = inspect.getsource(func).strip()
-        setuid = True if setuid is None else bool(setuid)
-
-        script = r"""
-#!/usr/bin/env python
-import os, sys, ctypes, resource, platform, stat
-from collections import OrderedDict
-try:
-    integer_types = int, long
-except NameError:
-    integer_types = int,
-exe   = bytes(%(executable)r)
-argv  = [bytes(a) for a in %(argv)r]
-env   = %(env)r
-
-os.chdir(%(cwd)r)
-
-environ = getattr(os, 'environb', os.environ)
-
-if env is not None:
-    env = OrderedDict((bytes(k), bytes(v)) for k,v in env)
-    os.environ.clear()
-    environ.update(env)
-else:
-    env = environ
-
-def is_exe(path):
-    return os.path.isfile(path) and os.access(path, os.X_OK)
-
-PATH = environ.get(b'PATH',b'').split(os.pathsep.encode())
-
-if os.path.sep.encode() not in exe and not is_exe(exe):
-    for path in PATH:
-        test_path = os.path.join(path, exe)
-        if is_exe(test_path):
-            exe = test_path
-            break
-
-if not is_exe(exe):
-    sys.stderr.write('3\n')
-    sys.stderr.write("{!r} is not executable or does not exist in $PATH: {!r}".format(exe,PATH))
-    sys.exit(-1)
-
-if not %(setuid)r:
-    PR_SET_NO_NEW_PRIVS = 38
-    result = ctypes.CDLL('libc.so.6').prctl(PR_SET_NO_NEW_PRIVS, 1, 0, 0, 0)
-
-    if result != 0:
-        sys.stdout.write('3\n')
-        sys.stdout.write("Could not disable setuid: prctl(PR_SET_NO_NEW_PRIVS) failed")
-        sys.exit(-1)
-
-try:
-    PR_SET_PTRACER = 0x59616d61
-    PR_SET_PTRACER_ANY = -1
-    ctypes.CDLL('libc.so.6').prctl(PR_SET_PTRACER, PR_SET_PTRACER_ANY, 0, 0, 0)
-except Exception:
-    pass
-
-# Determine what UID the process will execute as
-# This is used for locating apport core dumps
-suid = os.getuid()
-sgid = os.getgid()
-st = os.stat(exe)
-if %(setuid)r:
-    if (st.st_mode & stat.S_ISUID):
-        suid = st.st_uid
-    if (st.st_mode & stat.S_ISGID):
-        sgid = st.st_gid
-
-if sys.argv[-1] == 'check':
-    sys.stdout.write("1\n")
-    sys.stdout.write(str(os.getpid()) + "\n")
-    sys.stdout.write(str(os.getuid()) + "\n")
-    sys.stdout.write(str(os.getgid()) + "\n")
-    sys.stdout.write(str(suid) + "\n")
-    sys.stdout.write(str(sgid) + "\n")
-    getattr(sys.stdout, 'buffer', sys.stdout).write(os.path.realpath(exe) + b'\x00')
-    sys.stdout.flush()
-
-for fd, newfd in {0: %(stdin)r, 1: %(stdout)r, 2:%(stderr)r}.items():
-    if newfd is None:
-        os.close(fd)
-    elif isinstance(newfd, (str, bytes)):
-        newfd = os.open(newfd, os.O_RDONLY if fd == 0 else (os.O_RDWR|os.O_CREAT))
-        os.dup2(newfd, fd)
-        os.close(newfd)
-    elif isinstance(newfd, integer_types) and newfd != fd:
-        os.dup2(fd, newfd)
-
-if not %(aslr)r:
-    if platform.system().lower() == 'linux' and %(setuid)r is not True:
-        ADDR_NO_RANDOMIZE = 0x0040000
-        ctypes.CDLL('libc.so.6').personality(ADDR_NO_RANDOMIZE)
-
-    resource.setrlimit(resource.RLIMIT_STACK, (-1, -1))
-
-# Attempt to dump ALL core file regions
-try:
-    with open('/proc/self/coredump_filter', 'w') as core_filter:
-        core_filter.write('0x3f\n')
-except Exception:
-    pass
-
-# Assume that the user would prefer to have core dumps.
-try:
-    resource.setrlimit(resource.RLIMIT_CORE, (-1, -1))
-except Exception:
-    pass
-
-%(func_src)s
-%(func_name)s(*%(func_args)r)
-
-""" % locals()  
-
-        if len(argv) > 0 and len(argv[0]) > 0:
-            script += r"os.execve(exe, argv, env) " 
-
-        # os.execve does not allow us to pass empty argv[0]
-        # Therefore we use ctypes to call execve directly
-        else:
-            script += r"""
-# Transform envp from dict to list
-env_list = [key + b"=" + value for key, value in env.items()]
-
-# ctypes helper to convert a python list to a NULL-terminated C array
-def to_carray(py_list):
-    py_list += [None] # NULL-terminated
-    return (ctypes.c_char_p * len(py_list))(*py_list)
-
-c_argv = to_carray(argv)
-c_env = to_carray(env_list)
-
-# Call execve
-libc = ctypes.CDLL('libc.so.6')
-libc.execve(exe, c_argv, c_env)
-
-# We should never get here, since we sanitized argv and env,
-# but just in case, indicate that something went wrong.
-libc.perror(b"execve")
-raise OSError("execve failed")
-""" % locals()
-
-        script = script.strip()
 
         self.debug("Created execve script:\n" + script)
 
@@ -1091,6 +945,7 @@ raise OSError("execve failed")
                 self.chmod('+x', tmpfile)
 
             self.info("Uploading execve script to %r" % tmpfile)
+            script = packing._encode(script)
             self.upload_data(script, tmpfile)
             return tmpfile
 
@@ -1182,12 +1037,14 @@ raise OSError("execve failed")
         if os.path.sep in program:
             return program
 
-        result = self.run('export PATH=$PATH:$PWD; command -v %s' % program).recvall().strip().decode()
+        program = packing._encode(program)
 
-        if ('/%s' % program) not in result:
+        result = self.system(b'export PATH=$PATH:$PWD; command -v ' + program).recvall().strip()
+
+        if (b'/' + program) not in result:
             return None
 
-        return result
+        return packing._decode(result)
 
     def system(self, process, tty = True, cwd = None, env = None, timeout = None, raw = True, wd = None):
         r"""system(process, tty = True, cwd = None, env = None, timeout = Timeout.default, raw = True) -> ssh_channel
@@ -1201,6 +1058,7 @@ raise OSError("execve failed")
         Return a :class:`pwnlib.tubes.ssh.ssh_channel` object.
 
         Examples:
+
             >>> s =  ssh(host='example.pwnme')
             >>> py = s.system('python3 -i')
             >>> _ = py.recvuntil(b'>>> ')
@@ -1270,6 +1128,7 @@ from ctypes import *; libc = CDLL('libc.so.6'); print(libc.getenv(%r))
         a TTY on the remote server.
 
         Examples:
+
             >>> s =  ssh(host='example.pwnme')
             >>> print(s.run_to_end('echo Hello; exit 17'))
             (b'Hello\n', 17)
@@ -1296,6 +1155,7 @@ from ctypes import *; libc = CDLL('libc.so.6'); print(libc.getenv(%r))
         Returns a :class:`pwnlib.tubes.ssh.ssh_connecter` object.
 
         Examples:
+
             >>> from pwn import *
             >>> l = listen()
             >>> s =  ssh(host='example.pwnme')
@@ -1506,14 +1366,14 @@ from ctypes import *; libc = CDLL('libc.so.6'); print(libc.getenv(%r))
         with open(local, 'wb') as fd:
             fd.write(data)
 
-    def _download_to_cache(self, remote, p):
+    def _download_to_cache(self, remote, p, fingerprint=True):
 
         with context.local(log_level='error'):
             remote = self.readlink('-f',remote)
         if not hasattr(remote, 'encode'):
             remote = remote.decode('utf-8')
 
-        fingerprint = self._get_fingerprint(remote)
+        fingerprint = fingerprint and self._get_fingerprint(remote) or None
         if fingerprint is None:
             local = os.path.normpath(remote)
             local = os.path.basename(local)
@@ -1535,7 +1395,7 @@ from ctypes import *; libc = CDLL('libc.so.6'); print(libc.getenv(%r))
 
         return local
 
-    def download_data(self, remote):
+    def download_data(self, remote, fingerprint=True):
         """Downloads a file from the remote server and returns it as a string.
 
         Arguments:
@@ -1543,6 +1403,7 @@ from ctypes import *; libc = CDLL('libc.so.6'); print(libc.getenv(%r))
 
 
         Examples:
+
             >>> with open('/tmp/bar','w+') as f:
             ...     _ = f.write('Hello, world')
             >>> s =  ssh(host='example.pwnme',
@@ -1556,7 +1417,7 @@ from ctypes import *; libc = CDLL('libc.so.6'); print(libc.getenv(%r))
 
         """
         with self.progress('Downloading %r' % remote) as p:
-            with open(self._download_to_cache(remote, p), 'rb') as fd:
+            with open(self._download_to_cache(remote, p, fingerprint), 'rb') as fd:
                 return fd.read()
 
     def download_file(self, remote, local = None):
@@ -1570,6 +1431,7 @@ from ctypes import *; libc = CDLL('libc.so.6'); print(libc.getenv(%r))
             local(str): The local filename to save it to. Default is to infer it from the remote filename.
         
         Examples:
+
             >>> with open('/tmp/foobar','w+') as f:
             ...     _ = f.write('Hello, world')
             >>> s =  ssh(host='example.pwnme',
@@ -1647,6 +1509,7 @@ from ctypes import *; libc = CDLL('libc.so.6'); print(libc.getenv(%r))
             remote(str): The filename to upload it to.
 
         Example:
+
             >>> s =  ssh(host='example.pwnme')
             >>> s.upload_data(b'Hello, world', '/tmp/upload_foo')
             >>> print(open('/tmp/upload_foo').read())
@@ -1769,6 +1632,7 @@ from ctypes import *; libc = CDLL('libc.so.6'); print(libc.getenv(%r))
         
 
         Examples:
+
             >>> with open('/tmp/foobar','w+') as f:
             ...     _ = f.write('Hello, world')
             >>> s =  ssh(host='example.pwnme',
@@ -1804,20 +1668,42 @@ from ctypes import *; libc = CDLL('libc.so.6'); print(libc.getenv(%r))
 
         return self.sftp.unlink(file)
 
-    def libs(self, remote, directory = None):
+    def libs(self, remote, directory = None, flatten = False):
         """Downloads the libraries referred to by a file.
 
         This is done by running ldd on the remote server, parsing the output
         and downloading the relevant files.
 
         The directory argument specified where to download the files. This defaults
-        to './$HOSTNAME' where $HOSTNAME is the hostname of the remote server."""
+        to './$HOSTNAME' where $HOSTNAME is the hostname of the remote server.
+
+        Arguments:
+            remote(str): Remote file path
+            directory(str): Output directory
+            flatten(bool): Flatten the file tree if True (defaults to False) and
+                ignore the remote directory structure. If there are duplicate
+                filenames, an error will be raised.
+        """
 
         libs = self._libs_remote(remote)
 
         remote = packing._decode(self.readlink('-f',remote).strip())
         libs[remote] = 0
 
+        if flatten:
+            basenames = dict()
+
+            # If there is a duplicate switch to unflattened download
+            for lib in libs:
+                name = os.path.basename(lib)
+
+                if name in basenames.values():
+                    duplicate = [key for key, value in basenames.items() if
+                                 value == name][0]
+                    self.error('Duplicate lib name: %r / %4r' % (lib, duplicate))
+
+                basenames[lib] = name
+
         if directory is None:
             directory = self.host
 
@@ -1828,7 +1714,8 @@ from ctypes import *; libc = CDLL('libc.so.6'); print(libc.getenv(%r))
         seen = set()
 
         for lib, addr in libs.items():
-            local = os.path.realpath(os.path.join(directory, '.' + os.path.sep + lib))
+            local = os.path.realpath(os.path.join(directory, '.' + os.path.sep \
+                    + (basenames[lib] if flatten else lib)))
             if not local.startswith(directory):
                 self.warning('This seems fishy: %r' % lib)
                 continue
@@ -1886,6 +1773,7 @@ from ctypes import *; libc = CDLL('libc.so.6'); print(libc.getenv(%r))
                 that all files in the "old" working directory should be symlinked.
 
         Examples:
+
             >>> s =  ssh(host='example.pwnme')
             >>> cwd = s.set_working_directory()
             >>> s.ls()
@@ -2148,6 +2036,57 @@ from ctypes import *; libc = CDLL('libc.so.6'); print(libc.getenv(%r))
 
         return self._aslr_ulimit
 
+    def _cpuinfo(self):
+        if self._cpuinfo_cache is None:
+            with context.quiet:
+                try:
+                    self._cpuinfo_cache = self.download_data('/proc/cpuinfo', fingerprint=False)
+                except PwnlibException:
+                    self._cpuinfo_cache = b''
+        return self._cpuinfo_cache
+
+    @property
+    def user_shstk(self):
+        """:class:`bool`: Whether userspace shadow stack is supported on the system.
+
+        Example:
+
+            >>> s = ssh("travis", "example.pwnme")
+            >>> s.user_shstk # doctest: +SKIP 
+            False
+        """
+        if self._user_shstk is None:
+            if self.os != 'linux':
+                self.warn_once("Only Linux is supported for userspace shadow stack checks.")
+                self._user_shstk = False
+
+            else:
+                cpuinfo = self._cpuinfo()
+
+                self._user_shstk = b' user_shstk' in cpuinfo
+        return self._user_shstk
+
+    @property
+    def ibt(self):
+        """:class:`bool`: Whether kernel indirect branch tracking is supported on the system.
+
+        Example:
+
+            >>> s = ssh("travis", "example.pwnme")
+            >>> s.ibt
+            False
+        """
+        if self._ibt is None:
+            if self.os != 'linux':
+                self.warn_once("Only Linux is supported for kernel indirect branch tracking checks.")
+                self._ibt = False
+
+            else:
+                cpuinfo = self._cpuinfo()
+
+                self._ibt = b' ibt ' in cpuinfo or b' ibt\n' in cpuinfo
+        return self._ibt
+
     def _checksec_cache(self, value=None):
         path = self._get_cachefile('%s-%s' % (self.host, self.port))
 
@@ -2184,7 +2123,15 @@ from ctypes import *; libc = CDLL('libc.so.6'); print(libc.getenv(%r))
             "ASLR:".ljust(10) + {
                 True: green("Enabled"),
                 False: red("Disabled")
-            }[self.aslr]
+            }[self.aslr],
+            "SHSTK:".ljust(10) + {
+                True: green("Enabled"),
+                False: red("Disabled")
+            }[self.user_shstk],
+            "IBT:".ljust(10) + {
+                True: green("Enabled"),
+                False: red("Disabled")
+            }[self.ibt],
         ]
 
         if self.aslr_ulimit:
